/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* global define, window, document, history */

/**
 * Init page module.
 *
 * @requires core/event
 * @requires core/systeminfo
 * @requires core/application
 * @requires views/main
 * @requires views/settings
 * @requires views/preview
 * @requires views/pictureSize
 * @requires views/pictureFormat
 * @requires views/recordingFormat
 * @namespace views/initPage
 * @memberof views
 */

define({
    name: 'views/initPage',
    requires: [
        'core/event',
        'core/systeminfo',
        'core/application',
        'views/main',
        'views/settings',
        'views/preview',
        'views/pictureSize',
        'views/pictureFormat',
        'views/recordingFormat'
    ],
    def: function viewsInitPage(req) {
        'use strict';

        /**
         * Core application module.
         *
         * @private
         * @type {Module}
         */
        var app = req.core.application,

            /**
             * Core event module.
             *
             * @private
             * @type {Module}
             */
            e = req.core.event,

            /**
             * Core systeminfo module.
             *
             * @private
             * @type {Module}
             */
            sysInfo = req.core.systeminfo;

        /**
         * Handles hardware keys tap event.
         *
         * @private
         * @param {Event} ev
         * @fires views.initPage.application.exit
         * @fires views.initPage.main.open
         * @fires views.initPage.settings.open
         */
        function onHardwareKeysTap(ev) {
            var keyName = ev.keyName,
                page = document.getElementsByClassName('ui-page-active')[0],
                pageid = (page && page.id) || '';

            if (keyName === 'back') {
                if (pageid === 'main') {
                    e.fire('application.exit');
                    window.setTimeout(function appCloseTimeout() {
                        app.exit();
                    }, 100);
                } else if (pageid === 'preview' || pageid === 'settings') {
                    e.fire('main.open');
                } else if (pageid === 'picture-size' ||
                    pageid === 'picture-format' ||
                    pageid === 'recording-format') {
                    e.fire('settings.open');
                } else {
                    history.back();
                }
            }
        }

        /**
         * Handles visibilitychange event.
         *
         * @private
         * @param {Event} ev
         * @fires views.initPage.visibility.change
         */
        function onVisibilityChange(ev) {
            e.fire('visibility.change', ev);
        }

        /**
         * Handles window unload event.
         *
         * @private
         * @fires views.initPage.application.exit
         */
        function onUnload() {
            e.fire('application.exit');
        }

        /**
         * Handler onLowBattery state.
         *
         * @private
         */
        function onLowBattery() {
            app.exit();
        }

        /**
         * Handles window blur event.
         *
         * @private
         * @fires views.initPage.application.state.background
         */
        function onBlur() {
            e.fire('application.state.background');
        }

        /**
         * Handles window focus event.
         *
         * @private
         * @fires views.initPage.application.state.foreground
         */
        function onFocus() {
            e.fire('application.state.foreground');
        }

        /**
         * Registers module event listeners.
         *
         * @private
         */
        function bindEvents() {
            document.addEventListener('tizenhwkey', onHardwareKeysTap);
            document.addEventListener('visibilitychange', onVisibilityChange);
            window.addEventListener('unload', onUnload);
            window.addEventListener('blur', onBlur);
            window.addEventListener('focus', onFocus);
            sysInfo.listenBatteryLowState();

            e.listeners({
                'core.systeminfo.battery.low': onLowBattery
            });
        }

        /**
         * Initializes module.
         *
         * @memberof views/initPage
         * @public
         */
        function init() {
            bindEvents();
            sysInfo.checkBatteryLowState();
        }

        return {
            init: init
        };
    }

});
